/*
 * Stonage: a puzzle game
 * Copyright (C) 2001 Luc Saillard <luc@saillard.org>
 *
 * Routine to start a new game, do transition between each turns ...
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

#include "stonage.h"

#include <SDL.h>
#if defined(HAVE_SDL_SDL_IMAGE_H)
#include <SDL/SDL_image.h>
#endif

#include "game.h"
#include "screen.h"
#include "level.h"
#include "image.h"

static int Game_Menu(void);
static int Game_PlayLevel(void);
static int Game_Run(struct t_level *lvl);

extern SDL_Surface *screen; /* Declared in main.c */

/*
 * Function: Game_Menu_Loop
 *
 * Description:
 *   This function manage the result of the main menu. With this menu, the user
 *   can choose to:
 *     * start a new level
 *     * change musik
 *     * quit the game ...
 *     ...
 * Input:
 * Output:
 */
void 
Game_Menu_Loop(void)
{
  int action;

  while (1)
   {
     switch(action=Game_Menu())
      {
       case 1:
	 Game_PlayLevel();
	 break;
       case 2:
	 trace("Want to quit");
	 return;
      }
   }
}

/*
 * Function: Game_PlayLevel
 *
 * Description:
 *   This launch the game level specified by (game.turns). When the user
 *   wins a new level is launched. When the user loose, the function draw
 *   a loose splash screen
 *
 * Input:
 * Output:
 *   return a code to indicate if the user have loose,won, ...
 */
static int 
Game_PlayLevel(void)
{
  struct t_level *lvl;
  char levelname[FILENAME_MAX];
  int action;


  /* Draw interscreen */

  
  while (1)
   {
     /* Draw a new level interscreen */

     
     /* Load Level */
     snprintf(levelname,FILENAME_MAX,"level%2.2d.lvl",game.level);
     lvl = Level_Load(levelname);
     if (!lvl)
      {
	trace("Can't load the level");
	return 0;
      }
     if (!Level_Sanity_Check(lvl))
      {
	trace("This level is corrupted");
	return 0;
      }

     /* Do a cross fading with the last screen */
      {
	SDL_Surface *lvl_screen;
	lvl_screen = SDL_CreateRGBSurface(SDL_HWSURFACE,screen->w,screen->h,
	    screen->format->BitsPerPixel, screen->format->Rmask, screen->format->Gmask, screen->format->Bmask, screen->format->Amask);
	Level_Draw2Screen(lvl,lvl_screen);
	Screen_Fade(screen,lvl_screen);
	SDL_FreeSurface(lvl_screen);
      }
     /* Run the game on this level */
     action = Game_Run(lvl);
     Level_Free(lvl);
     switch (action)
      {
       case 1:
	 /* Player Lose (Timeout) */
	 trace("You lose ... timeout\n");

	 return 0;
	 break;
       case 2:
	 /* Player want to quit */
	 trace("Is it too difficult for you ?\n");

	 return 0;
	 break;
       case 3:
	 /* Player win */
	 trace("Yo, your are a good man\n");

	 game.level++;
	 break;
       default:
	 trace("unknown Game_Run action %d",action);
      }
   }
}

/**
 *
 * Draw to screen the intro
 *
 */
void 
Game_SplashScreen()
{
  SDL_Surface *image;
  SDL_Event event;

  image = Image_Load("images/splash.jpg",1);
  if (image!=NULL)
   {
     SDL_FillRect(screen,NULL,0);
     SDL_Flip(screen);
     Screen_Fade(screen,image);

     /* wait for key or button */
     while (1) 
      {
	SDL_WaitEvent(&event);
	if (event.type == SDL_QUIT) 
	  Game_Quit();
	if (event.type == SDL_KEYUP || event.type == SDL_MOUSEBUTTONUP)
	  break;
      }
     SDL_FreeSurface(image);
   } /* if (image) */
}

/**
 * Function: Game_Menu
 *
 * Description:
 *   This function is responsible to draw the main menu, and wait an action
 *   choosen by the user. When an action occured, the function returns a code
 *   to indicate this choice.
 *
 * Input:
 * Output:
 *   return a code to indicate if the user have loose,won, ...
 */
static int
Game_Menu(void)
{
  int action;
  SDL_Surface *image=NULL;
  SDL_Event event;

  image = Image_Load("images/menu.jpg",1);
  if (image==NULL)
    Game_Quit();
  SDL_BlitSurface(image, NULL, screen, NULL);
  SDL_Flip(screen);
  SDL_FreeSurface(image);

  action=0;
  while (!action) 
   {
     SDL_WaitEvent(&event);
     if (event.type == SDL_QUIT) 
       Game_Quit();
     if (event.type == SDL_KEYUP)
      {
	switch(event.key.keysym.sym)
	  {
	   case SDLK_F1:
	     action=1;
	     break;
	   case SDLK_q:
	     action=2;
	     break;
	   default:
	     action=0;
	     break;
	  }
      }
   }
  return action;
}

/**
 *
 */
void
Game_Quit(void)
{
  if (screen)
    SDL_FreeSurface(screen);
  SDL_Quit();
  exit(0);
}

struct
{
  int keysym;
  int direction;
//  int (*fct_movedinoto)(struct t_level *l, enum movement_type direction);
//  int (*fct_movekinesistto)(struct t_level *l, enum movement_type direction);;
//  int (*fct_movesquareto)(struct t_level *l, enum movement_type direction);;
} static move_actions[] =
{
   {SDLK_DOWN,MOVE_DOWN},
   {SDLK_UP,MOVE_UP},
   {SDLK_RIGHT,MOVE_RIGHT},
   {SDLK_LEFT,MOVE_LEFT},
   {0,0}
};



/**
 *
 * This is the main loop function which play the game
 *
 */

static int
Game_Run(struct t_level *lvl)
{
  int action,we_have_moved,i;
  SDL_Event event;

  action=0;
  while (!action) 
   {
     SDL_WaitEvent(&event);
     if (event.type == SDL_QUIT) 
      {
	Level_Free(lvl);
	Game_Quit();
      }
     if (event.type == SDL_KEYUP)
      {
	we_have_moved=0;
	switch(event.key.keysym.sym)
	 {
	  case SDLK_q:
	    action=2;
	    break;
	  case SDLK_SPACE: /* Popup a carre to move a square */
	    switch (lvl->control)
	     {
	      case CONTROL_PLAYER:
		/* Now we control the select square */
		lvl->control=CONTROL_KINESIST;
		lvl->xkinesist=lvl->xdino;
		lvl->ykinesist=lvl->ydino;
		break;
	      case CONTROL_KINESIST:
		/* Find if a moveable square is under the carre rouge */
		if (Level_CanMoveSquare(lvl))
		 {
		   /* We test again, because if the block can move again,
		    * the control return to the kinesist */
		   if (Level_CanMoveSquare(lvl))
		     lvl->control=CONTROL_SQUARE;
		   else
		     lvl->control=CONTROL_KINESIST;
		 }
		else
		 {
		   /* Give control to the player */
		   lvl->control=CONTROL_PLAYER;
		 }
		break;
	      case CONTROL_SQUARE:
		lvl->control=CONTROL_PLAYER;

		break;
	     }
	    we_have_moved=1; /* Refresh the screen */
	    break;
	  default:
	    action=0;
	    for (i=0;move_actions[i].keysym != 0; i++)
	     {
	       if (event.key.keysym.sym == move_actions[i].keysym)
		{
		  switch (lvl->control)
		   {
		    case CONTROL_PLAYER:
		      we_have_moved=Level_MoveDinoTo(lvl,move_actions[i].direction);
		      break;
		    case CONTROL_KINESIST:
		      we_have_moved=Level_MoveKinesistTo(lvl,move_actions[i].direction);
		      break;
		    case CONTROL_SQUARE:
		      we_have_moved=Level_MoveSquareTo(lvl,move_actions[i].direction);
		      break;
		   }
		  break; /* Exit the for () */
		}
	     }
	 } /* End of switch(event.key.keysym.sym) */
	if (we_have_moved)
	 {
	   Level_DoPostMoveAction(lvl);
	   Level_Draw2Screen(lvl,screen);
	   SDL_Flip(screen);
	   if (lvl->control==CONTROL_PLAYER && Level_IsFinish(lvl))
	     action=3;
	 }
      }
   }
  return action;
}


